package fr.velossity.sample.device.impl;

import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Iterator;
import java.util.List;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleEvent;
import org.osgi.framework.BundleListener;

import fr.velossity.sample.device.Device;
import fr.velossity.sample.device.DeviceHandler;

/**
 * 
 * @author C. Saint-Marcel
 * 
 */
public class Activator implements BundleActivator, BundleListener {
		
	private Device myDevice = new MockDeviceImpl("mock1");
	
	private List deviceHandlers = new ArrayList();

	/**
	 * @see BundleActivator#start(BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		
		// REQUESTS bundles already installed
		Bundle[] bundles = context.getBundles();
		boolean found = false;
		if (bundles != null) {
			for (int i = 0; i < bundles.length; i++) {
				found = found | lookForDeviceHandler(bundles[i]);
			}
		}
		if (!found) {
			System.out.println("I found no device listener at that time");
		}
		// REGISTER itself as a BundleListener
		context.addBundleListener(this);

	}

	/**
	 * Notify to all requesters that the device is disappearing.
	 * @see BundleActivator#stop(BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		for (Iterator iterator = deviceHandlers.iterator(); iterator.hasNext();) {
			DeviceHandler deviceHandler = (DeviceHandler) iterator.next();
			deviceHandler.free(myDevice);
		}
		deviceHandlers = null;
	}

	public void bundleChanged(BundleEvent event) {
		if(event.getType() == BundleEvent.STARTED) {
			lookForDeviceHandler(event.getBundle());
		}
	}

	private boolean lookForDeviceHandler(Bundle aBundle) {
		if (aBundle.getState() == Bundle.ACTIVE) {
			Dictionary headers = aBundle.getHeaders();
			String clazzName = (String) headers.get("Class-Extension");
			if (clazzName != null) {
				// Load the class and instantiate it.
				Class clazz;
				try {
					clazz = aBundle.loadClass(clazzName);
					// LOADS the service provider
					DeviceHandler myDeviceHandler = (DeviceHandler) clazz.newInstance();
					// INVOKES the service
					myDeviceHandler.handle(myDevice);
					
					// Keeps it for next calls
					deviceHandlers.add(myDeviceHandler);
					return true;
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		return false;
	}

}
